#include <bits/stdc++.h>

#define MAX_N (1000)
#define MAX_C (1000 * 10)
#define INFTY (MAX_C * MAX_N)

#define IMPOSSIBLE (-2)
#define LIMITLESS (-1)

using namespace std;

typedef size_t node;

struct path {
    int w;
    bool mattel;
    node parent;
};

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream &in(cin);
ostream &out(cout);
#endif

void print_minpath(const vector<vector<path>> &SP, const int C, const node A) {
    int minsteps = 1;
    for (int steps = 1; steps < C; ++steps) {
        if (SP[steps][A].w < SP[minsteps][A].w)
            minsteps = steps;
    }

    list<node> path;
    path.push_front(A);
    for (auto steps = minsteps; steps > 0; --steps) {
        path.push_front(SP[steps][path.front()].parent);
    }

    out << path.size() << endl;
    for (const node x: path) {
        out << x << ' ';
    }
    out << endl;
}

int main() {

    int C, S;
    in >> C >> S;

    const node B = 0;
    const node A = C - 1;

    vector<pair<pair<node, node>, int>> E;
    E.reserve(2 * S);
    for (; S; --S) {
        int a, b, c;
        in >> a >> b >> c;

        E.push_back({{a, b}, c});
        E.push_back({{b, a}, c});
    }

    vector<bool> is_mattel(C, false);
    int Z;
    in >> Z;
    for (; Z; --Z) {
        int m;
        in >> m;

        is_mattel[m] = true;
    }

    // SP[s][u] : minimum path with exactly `s` steps from `B` to `u`
    vector<vector<path>> SP(C, vector<path>(C, {.w=INFTY, .mattel=false, .parent=MAX_N}));
    SP[0][B] = {.w=0, .mattel=is_mattel[B], .parent=B};

    for (int steps = 1; steps < C; ++steps) {
        for (const auto e: E) {
            const node from = e.first.first;
            const node to = e.first.second;
            const int w = e.second;
            const int attempt = SP[steps - 1][from].w + w;

            if (attempt <= SP[steps][to].w) {
                SP[steps][to] = {
                        .w=attempt,
                        .mattel=SP[steps - 1][from].mattel
                                || is_mattel[to]
                                || (attempt == SP[steps][to].w && SP[steps][to].mattel),
                        .parent=from
                };
            }
        }
    }

    // find the edge-wise shortest path from `B` to `A`
    // if it does not visit any MATTEL node, answer is LIMITLESS
    for (int steps = 1; steps < C; ++steps) {
        if (SP[steps][A].w == INFTY) continue;
        if (!SP[steps][A].mattel) {
            out << LIMITLESS << endl;
            print_minpath(SP, C, A);
            return 0;
        }
        break;
    }

    vector<pair<int, int>> ks;
    ks.reserve(C);

    for (int fs = 1; fs < C; ++fs) {
        if (SP[fs][A].w == INFTY) continue;
        if (SP[fs][A].mattel) continue;

        // SP[fs][A].w + fs * K < SP[ms][A].w + ms * K
        // ( fs - ms ) * K < SP[ms][Y].w - SP[fs][A].w

        // when ( fs - ms ) > 0:
        // K < ( SP[ms][A].w - SP[fs][A].w ) / ( fs - ms )

        // when ( fs - ms ) < 0:
        // K > ( SP[ms][A].w - SP[fs][A].w ) / ( fs - ms )

        int lb = 0LL, ub = INFTY;

        for (int ms = 1; ms < fs; ++ms) {
            if (SP[ms][A].w == INFTY) continue;
            if (!SP[ms][A].mattel) continue;
            const int num = SP[ms][A].w - SP[fs][A].w;
            const int den = fs - ms;
            const auto r = div(num, den);
            ub = min(ub, r.quot - (1 - min(r.rem, 1)));
        }
        for (int ms = fs + 1; ms < C; ++ms) {
            if (SP[ms][A].w == INFTY) continue;
            if (!SP[ms][A].mattel) continue;
            const int num = SP[ms][A].w - SP[fs][A].w;
            const int den = fs - ms;
            lb = max(lb, num / den);
        }

        if (lb < ub) {
            ks.emplace_back(ub, fs);
        }
    }

    const auto K = max_element(ks.cbegin(), ks.cend());
    if (K == ks.cend()) {
        out << IMPOSSIBLE << endl;
        print_minpath(SP, C, A);
        return 0;
    }

    out << K->first << endl;
    print_minpath(SP, C, A);
    return 0;
}
